def build_update_expression(kwargs: dict) -> str:
    update_expression = "set "
    for key in kwargs.keys():
        update_expression += f"#{key} = :{key}, "
    return update_expression[:-2]

def build_expression_attribute_names(kwargs: dict) -> dict:
    expression_attribute_names = {}
    for key in kwargs.keys():
        expression_attribute_names[f"#{key}"] = key
    return expression_attribute_names

def build_expression_attribute_values(kwargs: dict) -> dict:
    expression_attribute_values = {}
    for key, value in kwargs.items():
        expression_attribute_values[f":{key}"] = {
            'S': value
        }
    return expression_attribute_values


from enum import StrEnum

_TRANSMISIONS_AUTO = ['automatic', 'automatic_transmission', 'automatyczna']
_TRANSMISIONS_MANUAL = ['manual', 'manual_transmission', 'manualna']

class Transmisions(StrEnum):
    AUTO = 'automatic'
    MANUAL = 'manual'

    @staticmethod
    def to_common(value: str) -> 'Transmisions':
        value = value.lower()
        if value in _TRANSMISIONS_MANUAL:
            return Transmisions.MANUAL
        
        if value in _TRANSMISIONS_AUTO:
            return Transmisions.AUTO
        
        raise ValueError(f'Invalid value: {value}')


class FuelTypes(StrEnum):
    PETROL = 'petrol'
    PETROL_CNG = 'petrol_cng'
    PETROL_LPG = 'petrol_lpg'
    DIESEL = 'diesel'
    ELECTRIC = 'electric'
    ETHANOL = 'ethanol'
    HYBRID = 'hybrid'
    PLUGIN_HYBRID = 'plugin_hybrid'
    HYDROGEN = 'hydrogen'

    @staticmethod
    def to_common(value: str) -> 'FuelTypes':
        value = value.lower().replace('-', '_')
        enum_value = FuelTypes.__members__.get(value.upper())

        if enum_value:
            return enum_value
        
        if _is_exceptional(value):
            return _exceptional_value_to_common(value)

        raise ValueError(f'Invalid value: {value}')
    

def _is_exceptional(value: str) -> bool:
    return value in ['petrol_lpg', 'lpg_petrol', 'lpg', 'cng', 'hybrid']

def _exceptional_value_to_common(value: str) -> FuelTypes:
    is_lpg = 'lpg' in value
    if is_lpg:
        return FuelTypes.PETROL_LPG
    
    is_hybrid_plugin = 'hybrid' in value
    if is_hybrid_plugin:
        return FuelTypes.PLUGIN_HYBRID
    
    is_cng = 'cng' in value
    if is_cng:
        return FuelTypes.PETROL_CNG
    
    raise ValueError(f'Invalid exceptional value: {value}')


from cs_ai_common.rds.db_connection import RdsDbConnection
from sqlalchemy.orm import sessionmaker


class DbSession:
    _connection: RdsDbConnection

    def __init__(self, connection: RdsDbConnection):
        self._connection = connection

    def get_session(self):
        Session = sessionmaker(bind=self._connection.get_engine())
        return Session()



import json
from typing import Dict, Optional
import boto3
from botocore.exceptions import ClientError

def try_get_object(key: str, bucket: str) -> Optional[Dict]:
    try:
        s3 = boto3.client('s3')
        response = s3.get_object(Bucket=bucket, Key=key)
        return json.loads(response["Body"].read().decode("utf-8"))
    except ClientError as e:
        if e.response["Error"]["Code"] == "NoSuchKey":
            return None
        raise e
    
def get_bucket_name_from_arn(arn: str) -> str:
    return arn[13:] if arn is not None else None